<?php
/**
 * Custom template tags for this theme.
 *
 * Eventually, some of the functionality here could be replaced by core features.
 *
 * @package MagazineX
 * @since   1.0.0
 */

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

if ( ! function_exists( 'wp_body_open' ) ) :

	/**
	 * Shim for sites older than 5.2.
	 *
	 * @link https://core.trac.wordpress.org/ticket/12563
	 */
	function wp_body_open() {
		do_action( 'wp_body_open' );
	}
endif;

if ( ! function_exists( 'magazinex_page_title' ) ) :

	/**
	 * Get the title.
	 *
	 * Returns the title of a blog page, archive page etc.
	 */
	function magazinex_page_title() {

		// Blog page.
		if ( is_singular() ) {
			the_title();
		} elseif ( is_search() ) { // Search page.
			/* translators: %s: search query. */
			printf( esc_html__( 'Search Results for: %s', 'magazinex' ), '<span>' . get_search_query() . '</span>' );
		} elseif ( is_home() ) {
			echo wp_kses_post( get_the_title( get_option( 'page_for_posts', true ) ) );
		} elseif ( is_author() ) {
			the_author();
		} elseif ( is_category() || is_tag() || is_tax() ) {
			single_term_title( '' );
		} elseif ( is_archive() ) {
			the_archive_title();
		}
	}
endif;

if ( ! function_exists( 'magazinex_archive_description' ) ) :

	/**
	 * Output the archive description.
	 *
	 * @since 1.0.0
	 */
	function magazinex_archive_description() {

		the_archive_description( '<div class="mzx-page-header-description">', '</div>' );

	}
endif;

if ( ! function_exists( 'magazinex_entry_meta_date' ) ) :

	/**
	 * Prints HTML with meta information for the current post-date/time.
	 */
	function magazinex_entry_meta_date() {

		$time_string = '<time class="entry-date published updated" datetime="%1$s">%2$s</time>';

		$time_string = sprintf(
			$time_string,
			esc_attr( get_the_date( DATE_W3C ) ),
			esc_html( get_the_date() ),
			esc_attr( get_the_modified_date( DATE_W3C ) ),
			esc_html( get_the_modified_date() )
		);

		$allowed_time_html = array(
			'time' => array(
				'datetime' => true,
				'class'    => true,
			),
		);

		echo '<span class="post-date">' . wp_kses( magazinex_get_icon( 'calender', false ), MagazineX_SVG_Icons::$allowed_html ) . '<a href="' . esc_url( get_permalink() ) . '" rel="bookmark">' . wp_kses( $time_string, $allowed_time_html ) . '</a></span>';
	}
endif;

if ( ! function_exists( 'magazinex_entry_meta_author' ) ) :

	/**
	 * Prints HTML with meta information for the current author.
	 */
	function magazinex_entry_meta_author() {
		echo '<span class="post-author"><span class="screen-reader-text">' . esc_html__( 'Posted by', 'magazinex' ) . '</span>' . get_avatar( get_the_author_meta( 'ID' ), 32 ) . '<span><span class="author vcard"><a class="url fn n" href="' . esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ) . '">' . esc_html( get_the_author() ) . '</a></span></span></span>';
	}
endif;

if ( ! function_exists( 'magazinex_entry_category' ) ) :

	/**
	 * Prints HTML with meta information for the categories assigned.
	 *
	 * Category Color for widgets and other
	 *
	 * @param bool $echo Boolean value to echo or just return.
	 *
	 * @return mixed
	 */
	function magazinex_entry_category( $echo = true ) {

		global $post;
		$categories = get_the_category();
		$separator  = '&nbsp;';
		$output     = '';

		if ( $categories ) {
			$output .= '<div class="post-categories">';

			foreach ( $categories as $category ) {
				$primary_color = get_theme_mod( 'magazinex_primary_color', '#BA81FF' );
					$output   .= '<a href="' . get_category_link( $category->term_id ) . '" style="color:' . $primary_color . '; background-color: ' . MagazineX_Utils::adjust_color_opacity( $primary_color, 10 ) . '" rel="category tag">' . $category->cat_name . '</a>';
			}

			$output .= '</div>';

			if ( $echo ) {
				echo trim( $output, $separator ); // phpcs:ignore WordPress.XSS.EscapeOutput.OutputNotEscaped
			} else {
				return trim( $output, $separator );
			}
		}
	}
endif;

if ( ! function_exists( 'magazinex_entry_tag' ) ) :

	/**
	 * Prints HTML with meta information for the tag assigned.
	 */
	function magazinex_entry_tag() {

		// Hide category and tag text for pages.
		if ( 'post' === get_post_type() && get_the_tag_list() ) {
			echo '<span class="post-tags">' . wp_kses( magazinex_get_icon( 'tag', false ), MagazineX_SVG_Icons::$allowed_html ) . wp_kses(
				get_the_tag_list( '' ),
				array(
					'a' => array(
						'href' => true,
						'rel'  => true,
					),
				)
			) . '</span>';
		}
	}
endif;

if ( ! function_exists( 'magazinex_entry_comments' ) ) :

	/**
	 * Prints HTML with meta information for the comments.
	 */
	function magazinex_entry_comments() {

		if ( ! post_password_required() && ( comments_open() || get_comments_number() ) ) {
			$comment_icon  = magazinex_get_icon( 'comment', false );
			$comment_html  = '<span class="comments-link">';
			$comment_html .= '<a href="' . get_comments_link() . '">' . $comment_icon . get_comments_number() . '</a>';
			$comment_html .= '</span>';

			echo $comment_html;
		}
	}
endif;

if ( ! function_exists( 'magazinex_entry_thumbnail' ) ) :

	/**
	 * Displays an optional post thumbnail.
	 *
	 * Wraps the post thumbnail in an anchor element on index views, or a div
	 * element when on single views.
	 */
	function magazinex_entry_thumbnail() {

		if ( post_password_required() || is_attachment() || ! has_post_thumbnail() ) {
			return;
		}

		global $post;

		?>
		<div class="entry-thumbnail">
			<?php
			if ( is_singular() ) :
				the_post_thumbnail();
			else :
				?>
				<a class="entry-thumbnail__link" href="<?php the_permalink(); ?>" aria-hidden="true" tabindex="-1">
					<?php
					the_post_thumbnail(
						'full',
						array(
							'alt' => the_title_attribute(
								array(
									'echo' => false,
								)
							),
						)
					);
					?>
				</a>
			<?php endif; // End is_singular(). ?>
		</div><!-- .entry-thumbnail -->
		<?php
	}
endif;

if ( ! function_exists( 'magazinex_comments' ) ) :

	/**
	 * Prints comment form and comments in single post and page.
	 */
	function magazinex_comments() {

		// If comments are open or we have at least one comment, load up the comment template.
		if ( comments_open() || get_comments_number() ) :
			comments_template();
		endif;
	}
endif;

if ( ! function_exists( 'magazinex_breadcrumb' ) ) :

	/**
	 * Get breadcrumb markup.
	 *
	 * @param array $args Array for breadcrumb before and after container.
	 */
	function magazinex_breadcrumb( $args = array() ) {

		$args = wp_parse_args(
			$args,
			array(
				'container_before' => '',
				'container_after'  => '',
			)
		);

		echo wp_kses_post( $args['container_before'] );

		magazinex_breadcrumb_trail(
			array(
				'show_browse' => false,
			)
		);

		echo wp_kses_post( $args['container_after'] );
	}
endif;

if ( ! function_exists( 'magazinex_post_navigation' ) ) :

	/**
	 * Template part post-navigation.
	 *
	 * @return void
	 */
	function magazinex_post_navigation() {
		get_template_part( 'template-parts/entry/entry', 'post-navigation' );
	}
endif;

if ( ! function_exists( 'magazinex_numbered_pagination' ) ) :
	function magazinex_numbered_pagination() {
		?>

		<div class="mzx-numbered-pagination">
			<?php
			$args = array(
				'type'      => 'list',
				'prev_text' => '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d="M8.293 12.707a1 1 0 010-1.414l6-6a1 1 0 111.414 1.414L10.414 12l5.293 5.293a1 1 0 01-1.414 1.414l-6-6z"/></svg>',
				'next_text' => '<svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24"><path d="M15.707 11.293a1 1 0 010 1.414l-6 6a1 1 0 01-1.414-1.414L13.586 12 8.293 6.707a1 1 0 011.414-1.414l6 6z" /></svg>',
			);

			the_posts_pagination( $args );
			?>
		</div>
		<?php
	}
endif;
